@@ -93,9 +93,9 @@ equivalent:
 
 Suppose that an antibiotic kills off half of the population for the
 first two species, but doesn't harm the third species. Then the
-perturbation vector would be as follows
+perturbation vector, after closure, would be as follows:
 
->>> antibiotic = np.array([1./2, 1./2, 1])
+>>> antibiotic = closure(np.array([1./2, 1./2, 1]))
 
 And the resulting perturbation would be
 
@@ -108,7 +108,7 @@ import numpy as np
 import scipy.stats
 
 
-def closure(mat):
+def closure(mat, *, out=None):
     """
     Performs closure to ensure that all elements add up to 1.
 
@@ -118,6 +118,10 @@ def closure(mat):
        a matrix of proportions where
        rows = compositions
        columns = components
+    out : array_like or None, optional
+        A location into which the result is stored. If provided, it must have
+        a shape that the inputs broadcast to. If not provided or None, a
+        freshly-allocated array is returned.
 
     Returns
     -------
@@ -145,14 +149,16 @@ def closure(mat):
 
     """
     mat = np.atleast_2d(mat)
+    if out is not None:
+        out = np.atleast_2d(out)
     if np.any(mat < 0):
         raise ValueError("Cannot have negative proportions")
     if mat.ndim > 2:
         raise ValueError("Input matrix can only have two dimensions or less")
-    if np.all(mat == 0, axis=1).sum() > 0:
+    norm = mat.sum(axis=1, keepdims=True)
+    if np.any(norm == 0):
         raise ValueError("Input matrix cannot have rows with all zeros")
-    mat = mat / mat.sum(axis=1, keepdims=True)
-    return mat.squeeze()
+    return np.divide(mat, norm, out=out).squeeze()
 
 
 def multiplicative_replacement(mat, delta=None):
@@ -169,6 +175,7 @@ def multiplicative_replacement(mat, delta=None):
        a matrix of proportions where
        rows = compositions and
        columns = components
+       each composition (row) must add up to unity (see :ref:`closure()`)
     delta: float, optional
        a small number to be used to replace zeros
        If delta is not specified, then the default delta is
@@ -207,7 +214,6 @@ def multiplicative_replacement(mat, delta=None):
            [ 0.0625,  0.4375,  0.4375,  0.0625]])
 
     """
-    mat = closure(mat)
     z_mat = (mat == 0)
 
     num_feats = mat.shape[-1]
@@ -248,10 +254,12 @@ def perturb(x, y):
         a matrix of proportions where
         rows = compositions and
         columns = components
+        each composition (row) must add up to unity (see :ref:`closure()`)
     y : array_like, float
         a matrix of proportions where
         rows = compositions and
         columns = components
+        each composition (row) must add up to unity (see :ref:`closure()`)
 
     Returns
     -------
@@ -269,8 +277,8 @@ def perturb(x, y):
     array([ 0.0625,  0.1875,  0.5   ,  0.25  ])
 
     """
-    x, y = closure(x), closure(y)
-    return closure(x * y)
+    z = np.multiply(x, y)
+    return closure(z, out=z)
 
 
 def perturb_inv(x, y):
@@ -298,10 +306,12 @@ def perturb_inv(x, y):
         a matrix of proportions where
         rows = compositions and
         columns = components
+        each composition (row) must add up to unity (see :ref:`closure()`)
     y : array_like
         a matrix of proportions where
         rows = compositions and
         columns = components
+        each composition (row) must add up to unity (see :ref:`closure()`)
 
     Returns
     -------
@@ -318,8 +328,8 @@ def perturb_inv(x, y):
     >>> perturb_inv(x,y)
     array([ 0.14285714,  0.42857143,  0.28571429,  0.14285714])
     """
-    x, y = closure(x), closure(y)
-    return closure(x / y)
+    z = np.divide(x, y)
+    return closure(z, out=z)
 
 
 def power(x, a):
@@ -346,6 +356,7 @@ def power(x, a):
         a matrix of proportions where
         rows = compositions and
         columns = components
+        each composition (row) must add up to unity (see :ref:`closure()`)
     a : float
         a scalar float
 
@@ -364,8 +375,8 @@ def power(x, a):
     array([ 0.23059566,  0.25737316,  0.26488486,  0.24714631])
 
     """
-    x = closure(x)
-    return closure(x**a).squeeze()
+    y = np.power(x, a)
+    return closure(y, out=y)
 
 
 def inner(x, y):
@@ -385,10 +396,12 @@ def inner(x, y):
         a matrix of proportions where
         rows = compositions and
         columns = components
+        each composition (row) must add up to unity (see :ref:`closure()`)
     y : array_like
         a matrix of proportions where
         rows = compositions and
         columns = components
+        each composition (row) must add up to unity (see :ref:`closure()`)
 
     Returns
     -------
@@ -404,8 +417,6 @@ def inner(x, y):
     >>> inner(x, y)  # doctest: +ELLIPSIS
     0.2107852473...
     """
-    x = closure(x)
-    y = closure(y)
     a, b = clr(x), clr(y)
     return a.dot(b.T)
 
@@ -437,6 +448,7 @@ def clr(mat):
        a matrix of proportions where
        rows = compositions and
        columns = components
+       each composition (row) must add up to unity (see :ref:`closure()`)
 
     Returns
     -------
@@ -452,7 +464,6 @@ def clr(mat):
     array([-0.79451346,  0.30409883,  0.5917809 , -0.10136628])
 
     """
-    mat = closure(mat)
     lmat = np.log(mat)
     gm = lmat.mean(axis=-1, keepdims=True)
     return (lmat - gm).squeeze()
@@ -482,6 +493,7 @@ def clr_inv(mat):
        a matrix of real values where
        rows = transformed compositions and
        columns = components
+       each composition (row) must add up to unity (see :ref:`closure()`)
 
     Returns
     -------
@@ -497,7 +509,8 @@ def clr_inv(mat):
     array([ 0.21383822,  0.26118259,  0.28865141,  0.23632778])
 
     """
-    return closure(np.exp(mat))
+    emat = np.exp(mat)
+    return closure(emat, out=emat)
 
 
 def ilr(mat, basis=None, check=True):
@@ -528,11 +541,10 @@ def ilr(mat, basis=None, check=True):
        a matrix of proportions where
        rows = compositions and
        columns = components
-
+       each composition (row) must add up to unity (see :ref:`closure()`)
     basis: numpy.ndarray, float, optional
         orthonormal basis for Aitchison simplex
         defaults to J.J.Egozcue orthonormal basis.
-
     check: bool
         Specifies if the basis is orthonormal.
 
@@ -551,7 +563,6 @@ def ilr(mat, basis=None, check=True):
     the dimensions of the basis needs be `D-1 x D`, where rows represent
     basis vectors, and the columns represent proportions.
     """
-    mat = closure(mat)
     if basis is None:
         basis = clr_inv(_gram_schmidt_basis(mat.shape[-1]))
     else:
@@ -593,6 +604,7 @@ def ilr_inv(mat, basis=None, check=True):
        a matrix of transformed proportions where
        rows = compositions and
        columns = components
+       each composition (row) must add up to unity (see :ref:`closure()`)
 
     basis: numpy.ndarray, float, optional
         orthonormal basis for Aitchison simplex
@@ -660,7 +672,7 @@ def alr(mat, denominator_idx=0):
        a matrix of proportions where
        rows = compositions and
        columns = components
-
+       each composition (row) must add up to unity (see :ref:`closure()`)
     denominator_idx: int
        the index of the column (2D-matrix) or position (vector) of
        `mat` which should be used as the reference composition. By default
@@ -680,7 +692,6 @@ def alr(mat, denominator_idx=0):
     >>> alr(x)
     array([ 1.09861229,  1.38629436,  0.69314718])
     """
-    mat = closure(mat)
     if mat.ndim == 2:
         mat_t = mat.T
         numerator_idx = list(range(0, mat_t.shape[0]))
@@ -773,6 +784,7 @@ def center(mat):
        a matrix of proportions where
        rows = compositions
        columns = components
+       each composition (row) must add up to unity (see :ref:`closure()`)
 
     Returns
     -------
@@ -788,9 +800,8 @@ def center(mat):
     array([ 0.14854315,  0.25728427,  0.29708629,  0.29708629])
 
     """
-    mat = closure(mat)
     cen = scipy.stats.gmean(mat, axis=0)
-    return closure(cen)
+    return closure(cen, out=cen)
 
 
 def centralize(mat):
@@ -802,6 +813,7 @@ def centralize(mat):
        a matrix of proportions where
        rows = compositions and
        columns = components
+       each composition (row) must add up to unity (see :ref:`closure()`)
 
     Returns
     -------
@@ -818,7 +830,6 @@ def centralize(mat):
            [ 0.32495488,  0.18761279,  0.16247744,  0.32495488]])
 
     """
-    mat = closure(mat)
     cen = scipy.stats.gmean(mat, axis=0)
     return perturb_inv(mat, cen)
 
